#!/usr/bin/python

#
#	R e p l y . p y
#

#	Created on June 25, 2005 by Simon Brown
#
#	Handles a message, sent to it via a postfix alias, and sends out a user reply
#	if required and previously set up with start.py and not stopped by stop.py.

#	05/07/31	Started creating some of the foundation objects, dbReplyDate in particular.
#	05/08/02	Finished 1st pass on everything except extract_sender.
#	05/08/06	Converted message handling procedures into class. Based code for these
#				on some tests done in a seperate file.
#	05/08/07	First attempts at running, had to modify param's for class methods to get them to work.
#				Fixed a number of problems w/ var scopes.
#	05/08/09	Now include priority level in default message.
#	05/08/12	Edited the default message. Added the -i option to sendmail call.
#	05/08/14	Changed time between next reply to 14 hours from 20.
#	05/08/17	Researched autoreply loops. Should filter out precedence of list or bulk.
#				Should also not reply to our own autoreplies!
#				http://marc.merlins.org/netrants/autoresponders.txt
#	05/08/18	Added isBulkPrecedence. No longer send replies to bulk email.
#				Added msg.printVitals
#	05/08/23	Added missing new line after headers for default msg.
#	05/09/11	is_virus_alert: added so that we can stop replying to virus alert messages.
#				main: now close db before sending autoreply to get it closed ASAP.
#				main: stopped logging some actions to system.log
#	05/09/17	starting version 0.20
#				kAliasScript: new awk based script for adding messages to the reply queue.
#				replyQueue: renamed emailMsg class to replyQueue to reflect new functionality
#	05/09/18	get_next_msg: added method.
#				replyQueue class: modified the __init__ method
#				pop_open: added method.
#				pop_close: added method.
#				kDBReplyTimePath: made a "constant" instead of class field.
#				get_recipient: added method
#				main: changed the loop so that it now checks for more then one message at a time.
#				needsReply: print statement now rounds seconds to int, displays value in hours.
#	05/09/19	Tested out code in live system.
#				VERSION 0.20
#	05/09/20	starting version 0.30
#				isOneOfUs: added method to determine if message came from our domain.
#				get_msg_type: added to discern between messages needing an autoreply & autoreply activation requests.
#				main: moved some code just before call to send_reply into send_reply. main is now
#				main: checking for commands sent to autoreply (as opposed to msgs needing replies).
#	05/09/22	get_receiving: renamed from get_recipient, now utilizes both the header added by alias script
#	05/09/24	get_sending_account: replaces isOneOfUs -- we needed to get account to store autoreply text for anyway.
#				email_text: added in from pwMonitor
#				main: added sending of usage help message when an unknown command is received
#				get_sender_address: now check for IndexError in case we get a bogus From: address
#				autoreply_on: now send acknowledgement email
#				Each user can now have unique autoreply messages.
#				autoreply_off: created method.
#				Tested out code: everything working so far.
#	05/09/25	set_alias: split off from autoreply_off code.
#				autoreply_on: now adds alias postfix aliases file.
#	05/10/02	get_content: returns None if couldn't get message payload
#				autoreply_on: now handles a msgTxt value of None by giving default message.
#				pop_open: replaced for loop w/ a list comprehension
#	05/10/05	is_list_precedence: added so that we could also filter out list messages.
#				get_sender_address: found bug where an email address w/o name could return wrong address.
#	05/10/30	is_list_precedence: forgot open & close paren's for call to lower on get_sender_domain string.
#	05/11/15	Added some explanotory comments.
#				main: now exit if pop_open returns False (error)
#				send_reply: logging is now more specific @ what was done.
#	05/11/23	is_virus_alert was returning false negatives. now check From address instead of Return-path.
#				autoreply_off: now delete old reply text when autoreply is turned off.
#				is_spam: added method
#				main: now use is_spam method to avoid sending replies to msgs marked by spamassassin as spam.
#	05/12/15	get_sender_address: trying to return None.lower()
#				main: now check if senderAddress is None before trying to reply
#	05/12/18	get_sender_address: when validating email being too strict with '.' for short addresses
#				that use a '.' as part of account name.
#
#	NOTES:
#		Could simplify by deleting response database at end of each day?
#		Add full user name to autoreply message.
#		Use match to speed up processing of lists?
#		Add a throttle to pop_open by limiting max. # of msg ID's in a pass?
#		Have code purge reply time database records instead of seperate cron task?


import anydbm
import commands, os, sys
import email, mimetypes, poplib
import string, time

#
#	USER CONFIGURED PARAMETERS
#

# This should be a unique header name to discourage forgeries. Be careful if changing
# this after auto-replies are active: you will need to fix the alias file to match
# the new headers.
kAccountHeader = "X-YOURDOMAIN-Account"

# This is the default reply message text that will be used if the user doesn't give
# a custom message or uses an HTML message to turn on autoreply.
kDefaultMsg = \
	"I am currently out of the office and am unable to answer your message.\n" + \
	"\n" + \
	"Please contact someone else on my team if you need an immediate response.\n"

# The domain that we will be handling autoreplies for.
kDomain = "yourdomain.com"

# Setting this value to True will allow autoreplies to messages of Precedence: list,
# but only if they come from an address matching kDomain.
kListException = True

# Host name or IP address of POP server with autoreply account
kPOPAddress = "127.0.0.1"

# Name of the POP account that will be handling the auto-replies.
kQueueAccount = "autoreply"

# Password for accessing autoreply POP account
kQueuePassword = "autoreplypasswordhere"

# No. of seconds between checks for queued messages needing reply.
# Not currently used
kCheckInterval = 20            

# If we have recently sent out an autoreply for an account to a recipient recently
# we will skip sending additional replies. This setting determines how long we will
# wait (in seconds) between replies.
# Autoreplies every hours (14 hours X 60 min/hr X 60 sec/min)
gSecondsBetweenReplies = 14 * 60 * 60;

### OVERRIDING FOR TESTING TO 1 HOUR
#gSecondsBetweenReplies = 3600

# Command paths
kSendmailPath = "/usr/sbin/sendmail"
kPostaliasPath = "/usr/sbin/postalias"

# Email address of autoreply account.
kQueueAddress = kQueueAccount+'@'+kDomain

# Email address of virus alerts. Using the Mac OS 10.4 default here.
kVirusAlertAddress = "virusalert@"+kDomain

kAliasScript1 = ""","|/usr/bin/awk '{if($0)print$0;else{print""" + '\\"' + kAccountHeader + ': '
kAliasScript2 = """\\\\n\\"; exit}}' | /usr/sbin/sendmail -i """ + kQueueAddress + '"'

# NOTE: This database should be purged every month, if not sooner, to remove
# stale DB entries. This will cause a possibility that a reply will be sent
# sooner then gSecondsBetweenReplies, but the effect should be minimal if run
# by a cron task at something like midnight on Saturday.
kDBReplyTimePath = "/var/sibr.reply/SenderDates"

# Path to aliases file used to direct a copy of email to autoreply account.
kReplyAliases = '/var/sibr.reply/aliases'

# Path to message sent to valid users if they send an invalid command.
kUsageHelpMsg = '/usr/share/sibr.reply/usageHelpMsg.txt'

# Message sent when a user's autoreply has been turned on.
kActivateMsg = '/usr/share/sibr.reply/activateMsg.txt'

# Message sent when a user's autoreply has been turned off.
kDeactivateMsg = '/usr/share/sibr.reply/deactivateMsg.txt'

# This will refer to the reply text database object after initialization.
gReplyTextDB = False

# Using the users email account as its key, this db stores the text that should
# be used for that user's autoreply message.
kDBReplyTextPath = "/var/sibr.reply/Text"

# Key is user's account name. Stores date when the user's autoreply should expire.
# NOT CURRENTLY USED
gDBReplyExpiresPath = "/var/sibr.reply/Expirations"


#
#	l o g g e r
#

def logger (inMsg):
	"""Output the given message to the system log"""
	
	commands.getoutput ('logger "' + inMsg + '"')

#
#	e m a i l _ t e x t
#

def email_text (*inStrOrPath):
	"""Sends an email using sendmail. The input is one or more parameters, which may either
	be a string or a file path. File path parameters must begin with the "/" character. Will
	throw an IOError if there was a problem reading in the file."""
	
	def return_str_or_file (strOrPathItem):
	
		# If first char is forward slash then we'll treat the string as a file path.
		if strOrPathItem[0] == '/':
			# Attempt to read in the message header in the file we were given the path to.
			# open file w/ read-only access.
			f = open (strOrPathItem,'r')
			
			# Read in the text and close the file.
			strOrPathItem = f.read()
			f.close()
		
		# Make sure we terminate each line.
		if strOrPathItem [-1] != '\n':
			return strOrPathItem + "\n"
		else:
			return strOrPathItem

	# Open a connection to the sendmail command in the shell.
	#	-i  don't treat a . on a line as end of text
	#  -t  extract recipients from text of message.
	
	p = os.popen(kSendmailPath + ' -i -t', 'w')
	
	for strItem in inStrOrPath:
		# Read in file if necessary, then output string to the sendmail process.
		# If it was a file & there was an error reading it in we'll catch that here.
		try:
			x = p.write (return_str_or_file (strItem))

		except IOError:
			print 'Error: could not write to the command', strItem
			return False

	# Close our connection to the sendmail/shell process.
	exitCode = p.close()
	
	return exitCode == False


#
#       r e p l y Q u e u e 
#

class replyQueue:

	#
	#	_ _ i n i t _ _
	#
	
	def __init__ (self):

		self.msg = False			# placeholder for email object
		self.msgTxt = ""
		self.msgList = []
		self.curMsgID = -1
		self.pop = False			# placeholder for POP3 object
	
	#
	#	g e t _ n e x t _ m s g
	#
	
	def get_next_msg (self):
		"""set the current message to the next message in reply queue, returning True if succesful,
		False if there are no further messages or there was an error."""
	
		try:
			if self.curMsgID > 0:
				# Delete the previous message now that we are done with it.
				self.pop.dele(self.curMsgID)

		except:
			logger('reply.py: error while deleting msg '+str(self.curMsgID))
			print 'error while deleting msg', self.curMsgID
			return False

		if self.msgList != []:
			try:
				# Get the next message ID in queue list.
				self.curMsgID = self.msgList.pop()
				
				# Fetch the message as a list, one line per item (plus response string).
				self.msgTxt = self.pop.retr (self.curMsgID)
				
			except:
				print 'error while reading msg',self.curMsgID
				logger('reply.py: error while reading msg '+str(self.curMsgID))
				
			else:
				# Skip respone string in item[0], convert the raw text items first into a single string,
				# and then into into an email object.
				self.msg = email.message_from_string ('\n'.join(self.msgTxt[1]))
				return True
				
		else:
			self.curMsgID = -1
			return False

	
	#
	#	p o p _ o p e n
	#

	def pop_open (self):
		"""Create a new connection to the reply queue account on the POP server, loading in a list
		of all the current queued messages. If there weren't any queued messages or there was a
		problem opening the connection the message list will be empty."""
	
		self.msgList = []
		self.curMsgID = -1
		
		try:
			# Open a connection to the server.
			self.pop = poplib.POP3(kPOPAddress)
			
			# Authenticate ourselves.
			self.pop.apop (kQueueAccount,kQueuePassword)
		
		except:
			logger('reply.py: error while opening POP account')
			print "Error! Couldn't open POP account"
			print "Check account & password settings"
			return False

		try:
			# Get a list of messages from the POP server.
			rawMsgList = self.pop.list()

		except:
			logger('reply.py: error listing POP messages')
			print 'error listing POP messages'
			return False

		# For every message in list given by call to pop.list, split the message ID off from the
		# string and collect result into msgList.
		self.msgList = [ msgItem.split()[0] for msgItem in rawMsgList [1] ]
		
		return True

	#
	#	p o p _ c l o s e
	#
	
	def pop_close (self):
		"""Close the open POP connection and clear fields."""
	
		try:
			self.pop.quit()

		except:
			logger('reply.py: error while closing POP account')
			print 'error while closing POP account'
		
		self.pop = False
		self.msg = False
		self.curMsgID = -1

	#
	#	g e t _ c o n t e n t
	#

	def get_content (self):
		"""Get the body of email message"""

		theMsg = self.msg
		charset = ''
		
		if theMsg.is_multipart():
			for part in theMsg.walk():
				if part.get_content_type() == 'text/plain':
					body = part.get_payload()
					charset = part.get_param('charset')
					break
		else:
			body = theMsg.get_payload()
			charset = theMsg.get_param('charset')
			
		if charset:
			body = unicode(body,charset.lower())
			
		return body

	#
	#	g e t _ m s g _ t y p e
	#
	
	def get_msg_type (self):
		"""Determine whether this is a message requiring a reply, a request to turn on an autoreply,
		or a request to turn off an autoreply. We then return 'reply', 'on','off', or None."""
		
		# Is the header added by the alias script present?
		if self.msg.get (kAccountHeader,None) != None:
			# Only a message needing a reply would have the header.
			return 'reply'
		
		# The command will be in the subject.
		subjectStr = self.msg.get ('Subject','').lower().strip()
		
		if subjectStr == '':
			return None
		
		return subjectStr

	
	#
	#	g e t _ r e c e i v i n g _ a c c o u n t
	#
	
	def get_receiving_account (self):
		"""Get the recipient account that triggered the msg to be queued. Returns None if there
		is any ambiguity (ie, more then one recipient or recipient can't be determined)."""

		# Get the account header value added by awk script in alias file.
		account = self.msg.get (kAccountHeader,None)
		
		# if we there isn't an account header (added by alias script), then we need to find out
		# who the message was addressed to.
		
		# If it wasn't sent here as a message to be replied to (in which case the alias script
		# would of added the receiving account in the headers), then it may of been sent to the
		# autoreply account directly. (Note: we're not going to bother with checking for CC or
		# BCC entries.)
		if account == None:
			# Get all recipients of the queued message.
			addressStrs = string.lower (self.msg.get ('To',''))
			
			# Is the queue account address anywhere in the To: header?
			if addressStrs.find (kQueueAddress) >= 0:
				account = kQueueAccount
			else:
				account = None
			
		return account

	#
	#	g e t _ s e n d e r - a d d r e s s
	#
	
	def get_sender_address (self):
		"""Get the email address of the sender"""

		addr = None

		# Get the "From:" header value.
		fullAddr = self.msg.get ("From","")

		if fullAddr.find ('<') > 0 and fullAddr [-1] == '>':
			# The actual email address is enclosed between "<" and ">", possibly prefixed by the
			# sender's name. We'll strip these off to get just the email address.
			try:
				addr = fullAddr [string.find (fullAddr,"<")+1:-1]
			except:
				print "Error! Couldn't parse out email address between '<' and '>'."
				addr = None
		
		else:
			# Check that this has the basic components of an email address.
			if fullAddr.find('@') > 0 and fullAddr.find('.')>0:
				addr = fullAddr

		if addr != None:
			return addr.lower()
		else:
			return None


	#
	#	g e t _ s e n d e r _ d o m a i n
	#
	
	def get_sender_domain (self):
		"""Get the domain of the sender"""

		# Get the sender's email address w/ the name, if present, already stripped off.
		addr = self.get_sender_address()

		try:
			addr = addr [addr.index("@")+1:]

		except:
			addr = None

		return addr

	#
	#	i s B u l k P r e c e d e n c e
	#
	
	def isBulkPrecedence (self):
		"""Is the message marked with the bulk precedence level?"""

		# Get the "Precedence:" header value in all upper case.
		precedence = string.upper (self.msg.get ("Precedence",""))

		return precedence == "BULK"

	#
	#	i s _ l i s t _ p r e c e d e n c e
	#
	
	def is_list_precedence (self):
		"""Is the message marked with the list precedence level? However, will return False
		if kListException is True and the domain of the From address == kDomain."""

		# Get the "Precedence:" header value in all upper case.
		precedence = string.upper (self.msg.get ("Precedence",""))

		if precedence == "LIST":
			if kListException and (self.get_sender_domain().lower() == kDomain):
				print '- kListException made'
				return False
			print
			return True
		
		return False

	#
	#	g e t _ l o c a l _ a c c o u n t
	#
	
	def get_sending_account (self):
		"""Returns account name from our domain that sent the message in the queue. If the message wasn't from
		our domain then None is returned."""
		
		ourStr = '@' + kDomain
		
		returnPath = string.lower (self.msg.get ('Return-Path',''))
		
		try:
			return returnPath[1:returnPath.index (ourStr)]

		except:
			return None


	#
	#	i s _ s p a m
	#
	
	def is_spam (self):
	
		# Get the spam header added by Spamassassin, or NO if it doesn't exist.
		spamFlag = self.msg.get ('X-Spam-Flag','NO')
		return spamFlag == ('YES')

	#
	#	i s _ v i r u s  a l e r t
	#
	
	def is_virus_alert (self):
	
		# Who sent the message?
		fromAddr = string.lower (self.msg.get ('From',''))
		return fromAddr.endswith ('<'+kVirusAlertAddress+'>')

	#
	#	p r i n t V i t a l s
	#
	
	def printVitals (self):
		"""Print some basic info. about the message."""
		
		print "Subject:",self.msg.get ("Subject","")
		print "From:",self.msg.get ("From","")
		xs = self.msg.get ("To","")
		if xs != "": print "To:",xs
		xs = self.msg.get ("Cc","")
		if xs != "": print "Cc:",xs
		xs = self.msg.get ("Precedence","")
		if xs != "": print "Precedence:",xs
		xs = self.msg.get ("X-Spam-Status","")
		if xs != "": print "X-Spam-Status:",xs
		

#
#	d b R e p l y T i m e
#

class dbReplyTime:
	# Using a key that is a combination of the recipient's and the sender's email
	# address, this database tracks when the last autoreply was sent to the
	# recipient/sender pair.
	dbRef = False

	def __init__ (self):
		
		self.dbRef = anydbm.open (kDBReplyTimePath, 'c')

	def close (self):
		self.dbRef.close()

	#
	#	g e t D B K e y
	#
	
	def getDBKey (self,inUserAccount, inSenderAddress):
		"""Get the address pair formatted for use as a key in the database."""
		## Validate senderAddress???
		
		# Get max of 10 chars to use as recipient portion of key.
		# For the sender, we'll use 16 chars b/c we have to worry about the domain too.
		keyPair = inUserAccount[0:9] + ":" + inSenderAddress[0:15]

		return keyPair
	
	#
	#	g e t R e p l y T i m e
	#

	def getReplyTime (self,inKeyPair):
	
		try:
			# Look for record with matching account & sender address.
			# If it's successful, it will have the time (# of seconds since last epoch)
			# that the last reply was sent.
			lastReply = self.dbRef [inKeyPair]

		except KeyError:
			# We came here b/c the account & sender key does not exist,
			# which means that we have never sent a reply.
			lastReply = 0
	
		return lastReply

	#
	#	n e e d s R e p l y
	#
	
	def needsReply (self, inAccount, inSenderAddress):
		"""Return True if the given account and sender is due for a reply"""
		
		global gSecondsBetweenReplies
		
		try:
			# Look for record with matching account & sender address.
			# If it's successful, it will have the time (# of seconds since last epoch)
			# that the last reply was sent.
			key = self.getDBKey (inAccount, inSenderAddress)

			lastReply = self.getReplyTime (key)

			print "key="+key,"lastReply=",lastReply
	
		except KeyError:
			# We came here b/c the account & sender key does not exist,
			# which means that we have never sent a reply.
			return True
	
		# Current time - time of last reply (as string)
		secondsSinceReply = int (time.time() - int (lastReply))
	
		print "secondsSinceReply=",secondsSinceReply, "("+str(secondsSinceReply/3600)+" hours) gSecondsBetweenReplies=",gSecondsBetweenReplies

		# True if a new reply is due
		return (secondsSinceReply > gSecondsBetweenReplies)
	
	#
	#	s e t R e p l y T i m e
	#

	def setReplyTime (self,inAccount, inSenderAddress):
	
		# Create a key that is based on the user's account plus the sender's
		# email address. Then store the time that the reply was sent.
		self.dbRef [self.getDBKey (inAccount, inSenderAddress)] = str (int (time.time()))


#end class dbReplyTime	


#
#	a c t i v e _ a c c o u n t s
#

def active_accounts (inRecipients):
	
	foundList = []
	
	# Access the dates when a users auto-reply expires.
	dbReplyExpires = anydbm.open (gDBReplyExpiresPath, 'c')

	# get list of all accounts in database.
	activeList = dbReplyExpires.keys()

	dbReplyExpires.close()
	
	for account in inRecipients:
		if account in activeList:
			foundList.append (account)
	
	return foundList

#
#	s e t _ a l i a s
#

def set_alias (inTurnOn, inAccount):

	aliasesChanged = False

	# convenience string for speed.
	accountWithColon = inAccount+':'

	# Does the file exist?
	if not os.access (kReplyAliases,os.F_OK):
		try:
			f = open (kReplyAliases,'w')
		except IOError:
			print 'Error: couldn''t create the aliases file at',kReplyAliases
			return False
		aliases = []
		
	else:
		# The file exists. Attempt to open it for read/write access so we can read in the current
		# aliases and then write out the altered file.
		try:
			f = open (kReplyAliases,'r+')
		except IOError:
			print 'Error: couldn''t open the aliases file at',kReplyAliases
			return False

		# Read in the text, converting it into a list w/ one line per item
		aliases = f.read().splitlines()
		
		##f.close()
		##f = open(kReplyAliases,'w')

		# No matter whether we're turning an autoreply off or on, we are first going to
		# strip off any existing aliases.
		
		for line in aliases:
			# Does the line start with an alias to the account we want to turn off?
			if line.startswith (accountWithColon):
				# Remove the line containing alias directing email to autoreply account.
				aliases.remove (line)
				aliasesChanged = True
				# We won't break the loop here, we'ew going to keep looking for any stray aliases for this account.

	# Was this was a request to add an autoreply alias?
	if inTurnOn:
		# Construct our alias w/ an awk based script to add header with account name.
		aliases.append (inAccount+':\t'+inAccount+kAliasScript1+inAccount+kAliasScript2)

		# We'll need to run postalias after this change.
		aliasesChanged = True

	if aliasesChanged:
		# Need this seek before truncate to avoid having call to truncate just zero out previous text.
		f.seek (0)
		f.truncate (0)
		for line in aliases:
			print line
			f.write(line+'\n')
	
		# Be sure to close before calling postalias!
		f.close()
		
		# Update the postfix alias table.
		# How to error check this?
		st, tx = commands.getstatusoutput (kPostaliasPath+' hash:'+kReplyAliases)
		print 'postalias:',st,tx

	else:
		# Close the aliases file, even though we only read from it.
		f.close()

	return aliasesChanged
	
#
#	a u t o r e p l y _ o n
#

def autoreply_on (inAccount,msgTxt):

# check for valid reply text content
# add alias if not already there
# send message acknowledging what was done

	global gReplyTextDB
	
	print 'autoreply_on:',inAccount
	print msgTxt

	if msgTxt == None:
		msgTxt = kDefaultMsg

	gReplyTextDB [inAccount] = msgTxt
	
	print 'keys:',	gReplyTextDB
	print gReplyTextDB.keys()
	print

	# Setup an alias to redirect a copy of email to autoreply account.
	if set_alias (True, inAccount):			
	
		# Email acknowledgement of autoreply activation.
		email_text ('To: '+inAccount+'@'+kDomain,kActivateMsg,gReplyTextDB [inAccount])
	
	else:
		print "Error! Could not set up email alias for",inAccount

#
#	a u t o r e p l y _ o f f
#

def autoreply_off (inAccount):

	global gReplyTextDB

	print 'autoreply_off:',inAccount

	if set_alias (False,inAccount):
		try:
			# Delete the account's autoreply text, if present.
			del gReplyTextDB [inAccount]
		except:
			pass	

		# Email acknowledgement of autoreply deactivation.
		email_text ('To: '+inAccount+'@'+kDomain,kDeactivateMsg)

		print 'keys:',	gReplyTextDB
		print gReplyTextDB.keys()
		print

#
#	s e n d _ r e p l y
#

def send_reply (inAccount, inSender):

	global gReplyTextDB,gReplyTimeDB

	# Has this account not sent an autoreply to the sender recently?
	# Was there the expected header value?
	if inAccount != '' and gReplyTimeDB.needsReply (inAccount, inSender):

		#logger ("Reply: sending " + recipientAccount + " " + senderAddress)
		# Add/update timestamped database entry for this account & sender
		gReplyTimeDB.setReplyTime (inAccount, inSender)

		accountAddress = inAccount + "@" + kDomain

		msgTxt = "Subject: autoreply\n" + \
		"From: " + accountAddress + "\n" + \
		"To: " + inSender + "\n" + \
		"Precedence: bulk\n\n"

		try:
			msgTxt += gReplyTextDB [inAccount]
		
		except KeyError:
			msgTxt += kDefaultMsg

		print "autoreply to:",inSender
	 
		email_text (msgTxt)
	else:
		if inAccount == '':
			print "inAccount was empty - no reply sent"
		else:
			print "previous reply still in effect - no reply sent"


#
#	m a i n
#

def main():

	global gReplyTimeDB,gReplyTextDB
	
	#logger ("reply.py: starting up")
	#print "Start:", time.asctime()

	msgQueue = replyQueue()
	if not msgQueue.pop_open():
		# Couldn't check the autoreply account, so we'll bail here.
		sys.exit(1)

	# Create the reply time object.
	gReplyTimeDB = dbReplyTime ()

	# Access the dates when a users auto-reply expires. Any accounts listed
	# should also be ones with active auto-replies.
	gReplyTextDB = anydbm.open (kDBReplyTextPath, 'c')

	while msgQueue.get_next_msg():
		# Print info. on incoming msg to log.
		msgQueue.printVitals()

		# Get the address of sender of the message we may need to reply to.
		senderAddress = msgQueue.get_sender_address()

		recipientAccount = msgQueue.get_receiving_account()

		# we aren't goint to answer bulk, list, spam, or virus alert emails.
		if not msgQueue.isBulkPrecedence() \
		   and not msgQueue.is_list_precedence() \
		   and not msgQueue.is_virus_alert() \
		   and not msgQueue.is_spam() \
		   and senderAddress != None \
		   and recipientAccount != None:
		
			msgType = msgQueue.get_msg_type()
			
			if msgType == 'reply':
					
				# Send out the reply to sender's message using the current account.
				send_reply (recipientAccount, senderAddress)

			# It's not a reply, so it may be a request to turn on or off the autoreply.
			# We need to check that the message originated from our domain.
			
			else:
				localSenderAccount = msgQueue.get_sending_account()
				if localSenderAccount != None:
					
					# Activate/replace autoreply?
					if msgType == 'on':
						autoreply_on (localSenderAccount,msgQueue.get_content())

					# Deactivate the autoreply?
					elif msgType == 'off':
						autoreply_off (localSenderAccount)

					else:
						print 'Error! Unknown command type'
						email_text ('To: '+senderAddress,kUsageHelpMsg)

				else:
					print 'Warning! external message sent directly to autoreply account'
					# The message didn't originate from us & it isn't an autoreply, so we're
					# going to ignore it.
	
		# Put a blank lines between output for message actions
		print

	gReplyTextDB.close()
	gReplyTimeDB.close()
	msgQueue.pop_close()

	#print "reply.py: done",time.asctime()
	#print


main()
